package ch.il06.zeiterfassung.db;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Date;
import java.util.ResourceBundle;

import ch.il06.dbhelper.connectionpooling.DbConnectionPoolPair;
import ch.il06.dbhelper.connectionpooling.DbPoolPairManager;
import ch.il06.zeiterfassung.db.DbMethods.GetHolidayRanges;

/**
*
* @author ncaspar, mstutz, bdreher
*/

public class Db implements DbInterface {

	private static DbInterface instance = null;
	private static DbPoolPairManager con = null;
	final long MILLSECS_PER_DAY = 24 * 60 * 60 * 1000;

	private Db() {
		try {
			Class.forName("com.mysql.jdbc.Driver");
		} catch (ClassNotFoundException e) {
			// TODO Exception behandeln
			e.printStackTrace();
		}
		dbConnection();
	}

	private void dbConnection() {
		ResourceBundle rb = ResourceBundle.getBundle("ch.il06.zeiterfassung.properties.dbConnection");
		con = DbPoolPairManager.getInstance();
		DbConnectionPoolPair pool = new DbConnectionPoolPair(
					rb.getString("dbUrl"), 
					rb.getString("username"), 
					rb.getString("password"),
					rb.getString("autoCommit").equalsIgnoreCase("true"),
					Integer.parseInt(rb.getString("maxConnections")));
		con.addPool("standard", pool);
	}

	public static DbInterface getInstance() {
		if (instance == null) {
			instance = new Db();
		}
		return instance;
	}

	public Object call(DbCallable funktions) throws Exception{
		Connection c = con.getConnection("standard");
		try {
			return funktions.dbCall(c);
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		} finally {
			con.putConnection("standard", c);
		}
	}
	
	@Override
	public Group createGroup(String name, User master) throws Exception {
		Group group = (Group) call(new DbMethods().new CreateGroup(name, master));
		return group;
	}

	public Group getGroupById(int gid) throws Exception {
		Group group = (Group) call(new DbMethods().new GetGroupById(gid));
		return group;
	}

	public Role getRoleById(int rid) throws Exception {
		Role role = (Role) call(new DbMethods().new GetRoleById(rid));
		return role;
	}

	public Group getGroupByUserId(int uid) throws Exception {
		Group group = (Group) call(new DbMethods().new GetGroupByUserId(uid));
		return group;
	}

	public User getUserById(int uid) throws Exception {
		User user = (User) call(new DbMethods().new GetUserById(uid));
		return user;
	}

	@Override
	public UserCalendar getCalendarByApprentice(User user) {
		return new UserCalendar(this, user);
	}

	@Override
	public WorkDay[] getWorkDays(Date from, Date to, UserCalendar calendar) throws Exception {
		WorkDay[] workDay = (WorkDay[]) call(new DbMethods().new GetWorkDays(from, to, calendar));
		return workDay;
	}

	@Override
	public DayRange[] getOutfallDayRanges(Date from, Date to,
			UserCalendar calendar) throws Exception {
		DayRange[] outfallDayRange= (DayRange[]) call(new DbMethods().new GetOutfallDayRanges(from, to, calendar));
		return outfallDayRange;
	}

	@Override
	public GroupCalendar getCalendarByGroup(Group group) {
		return new GroupCalendar(this, group);
	}

	@Override
	public DayRange[] getGroupDayRanges(Date from, Date to,
			GroupCalendar calendar) throws Exception {
		DayRange[] dayRange= (DayRange[]) call(new DbMethods().new GetGroupDayRanges(from, to, calendar));
		return dayRange;
	}

	@Override
	public OfficialHolidayCalendar getOfficialHolidayCalendar() {
		return new OfficialHolidayCalendar(this);
	}

	@Override
	public DayRange[] getHolidayRanges(Date from, Date to,
			OfficialHolidayCalendar calendar) throws Exception {
		
		GetHolidayRanges hr =  new DbMethods().new GetHolidayRanges(from, to, calendar);
		DayRange[] dayRange= (DayRange[]) call(hr);
		return dayRange;
	}

	@Override
	@Deprecated
	public Day[] getDays(Date from, Date to, Calendar calendar) {
		throw new Error("Don't use this!");
	}

	@Override
	@Deprecated
	public void save(Day day, Calendar calendar) {
		throw new Error("Don't use this!");
	}

	@Override
	public void save(WorkDay day, UserCalendar calendar) throws Exception {
		call(new DbMethods().new Save(day, calendar));	
	}

	@Override
	public void visieren(Date fromday, Date today, UserCalendar calendar) throws Exception {
		call(new DbMethods().new Visieren(fromday, today, calendar));
	}

	@Override
	public void save(DayRange range, Calendar calendar) throws Exception {
		call(new DbMethods().new SaveDayRange(range, calendar));
	}

	@Override
	public User login(String pid, String password) throws Exception {
		User user= (User) call(new DbMethods().new Login(pid, password));
		return user;
	}

	@Override
	public Group[] searchGroup(String group, User lehrmeister) throws Exception {
		Group[] result= (Group[]) call(new DbMethods().new SearchGroup(group, lehrmeister));
		return result;
	}

	@Override
	public User[] searchUser(String pid, String name, String firstname,
			Group group) throws Exception {
		User[] user= (User[]) call(new DbMethods().new SearchUser(pid, name, firstname, group));
		return user;
	}

	@Override
	public User[] getUsersByRole(Role role) throws Exception {
		User[] user= (User[]) call(new DbMethods().new UserByRole(role));
		return user;
	}

	@Override
	public int save(User user) throws Exception {
		int i= (Integer) call(new DbMethods().new SaveUser(user));
		return i;
	}

	@Override
	public int save(Group group) throws Exception {
		int i= (Integer) call(new DbMethods().new SaveGroup(group));
		return i;
	}

	@Override
	public User[] getUserByInstructor(User instructor) throws Exception {
		User[] user= (User[]) call(new DbMethods().new GetUserByInstructor(instructor));
		return user;
	}
}