package ch.il06.zeiterfassung.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;

import ch.il06.dbhelper.connectionpooling.DbPoolPairManager;

/**
*
* @author mstutz
* @author ncaspar
*/
public class DbMethods {

	public class CreateGroup extends DbCallable {
		String name;
		User master;
		
		public CreateGroup(String name, User master) {
			this.name = name;
			this.master = master;
		}

		public Object dbCall(Connection c) throws SQLException {
			PreparedStatement preparedStatement = c.prepareStatement(
					"INSERT  gruppe SET Gruppenname = ?, Lehrmeister = ?",
					PreparedStatement.RETURN_GENERATED_KEYS);

			preparedStatement.setString(1, name);
			preparedStatement.setInt(2, master.getUid());

			ResultSet keys = preparedStatement.getGeneratedKeys();
			keys.next();
			int gid = keys.getInt(1);
			GetGroupById group = new GetGroupById(gid);
			return group.dbCall(c);
			
		}

	}
	
	public class GetGroupById extends DbCallable {
		int gid;
		
		public GetGroupById(int gid) {
			this.gid = gid;
		}

		public Object dbCall(Connection c) throws SQLException{
			PreparedStatement preparedStatement = c
			.prepareStatement("SELECT * FROM gruppe WHERE GID = ?");
			preparedStatement.setInt(1, gid);
			ResultSet rs = preparedStatement.executeQuery();
			if (!rs.next()) {
				return null;
			}
			return new Group(rs);
		}

	}
	
	public class GetRoleById extends DbCallable {
		int rid;
		
		public GetRoleById(int rid) {
			this.rid = rid;
		}

		public Object dbCall(Connection c) throws SQLException {
			PreparedStatement preparedStatement = c
			.prepareStatement("SELECT Rollenname FROM rolle WHERE RID = ?");
			preparedStatement.setInt(1, rid);
			ResultSet rs = preparedStatement.executeQuery();
			if (!rs.next()) {
				return null;
			}
			String name = rs.getString(1);
			if ("Lehrling".equals(name)) {
				return Role.Apprentice;
			} else if ("Lehrmeister".equals(name)) {
				return Role.Master;
			} else if ("Berufsbildner".equals(name)) {
				return Role.Instructor;
			}
			return null;
		}

	}
	
	public class GetGroupByUserId extends DbCallable {
		int uid;
		
		public GetGroupByUserId(int uid) {
			this.uid = uid;
		}

		public Object dbCall(Connection c) throws Exception {
			PreparedStatement preparedStatement = c
			.prepareStatement("SELECT * FROM usergruppenzuteilung, gruppe WHERE gruppe.GID = usergruppenzuteilung.GID AND usergruppenzuteilung.UID = ?");
			preparedStatement.setInt(1, uid);
			ResultSet rs = preparedStatement.executeQuery();
			if (!rs.next()) {
				return null;
			}
			Group g = new Group(rs);
			g.setLehrmeister((User) new GetUserById(rs.getInt("Lehrmeister")).dbCall(c));
			return g;
		}

	}
	
	public class GetUserById extends DbCallable {
		int uid;
		
		public GetUserById(int uid) {
			this.uid = uid;
		}

		public Object dbCall(Connection c) throws Exception {
			PreparedStatement preparedStatement = c
			.prepareStatement("SELECT * FROM user WHERE UID = ?");
			preparedStatement.setInt(1, uid);
			ResultSet rs = preparedStatement.executeQuery();
			if (!rs.next()) {
				return null;
			}
			User user = getUserByResultSet(rs, c);
			return user;
		}

	}
	
	public class GetWorkDays extends DbCallable {
		Date from;
		Date to;
		UserCalendar calendar;
		
		public GetWorkDays(Date from, Date to, UserCalendar calendar) {
			this.from = from;
			this.to = to;
			this.calendar = calendar;
		}
		
		private WorkDay[] getWorkDaysByResultSet(ResultSet rs) throws SQLException {
			if (!rs.next()) {
				return null;
			}
			ArrayList<WorkDay> al = new ArrayList<WorkDay>();
			do {
				WorkDay wd = new WorkDay(new Date(rs.getDate("Datum").getTime()));
				Worktime[] wt = new Worktime[2];
				wt[0] = new Worktime(rs.getInt("Von_Vormittag"), rs
						.getInt("Bis_Vormittag"));
				wt[1] = new Worktime(rs.getInt("Von_Nachmittag"), rs
						.getInt("Bis_Nachmittag"));
				wd.setWorktimes(wt);
				al.add(wd);
			} while (rs.next());
			WorkDay[] wa = new WorkDay[al.size()];
			return al.toArray(wa);
		}

		public Object dbCall(Connection c) throws SQLException {
			String sql = "SELECT TIME_TO_SEC(Von_Vormittag) AS Von_Vormittag, TIME_TO_SEC(Bis_Vormittag) AS Bis_Vormittag, TIME_TO_SEC(Von_Nachmittag) AS Von_Nachmittag, TIME_TO_SEC(Bis_Nachmittag) AS Bis_Nachmittag, Datum FROM arbeitstag WHERE UID = ?";
			if (from != null) {
				sql += " AND arbeitstag.Datum >= ?";
			}
			if (to != null) {
				sql += " AND arbeitstag.Datum <= ?";
			}
			PreparedStatement preparedStatement = c.prepareStatement(sql);
			preparedStatement.setInt(1, calendar.getUser().getUid());
			int i = 1;
			if (from != null) {
				java.sql.Date fromSql = new java.sql.Date(from.getTime());
				preparedStatement.setDate(++i, fromSql);
			}
			if (to != null) {
				java.sql.Date toSql = new java.sql.Date(to.getTime());
				preparedStatement.setDate(++i, toSql);
			}
			ResultSet rs = preparedStatement.executeQuery();
			return getWorkDaysByResultSet(rs);
		}
		
	}
	
	public class GetOutfallDayRanges extends DbCallable {
		Date from;
		Date to;
		UserCalendar calendar;
		
		public GetOutfallDayRanges(Date from, Date to, UserCalendar calendar) {
			this.from = from;
			this.to = to;
			this.calendar = calendar;
		}
		
		private DayRange[] getOutfallDayRangesByResultSet(ResultSet rs)
		throws SQLException {
			if (!rs.next()) {
				return null;
			}
			ArrayList<DayRange> al = new ArrayList<DayRange>();
			do {
				java.sql.Date from = rs.getDate("Von");
				java.sql.Date to = rs.getDate("Bis");
				Type type = getType(rs.getString("Typ"));
				al.add(new DayRange(new Date(from.getTime()), new Date(to
						.getTime()), rs.getInt("Anzahl_Stunden"), rs
						.getString("Bemerkungen"), type));
			} while (rs.next());
			DayRange[] wa = new DayRange[al.size()];
			return al.toArray(wa);
		}

		public Object dbCall(Connection c) throws SQLException {
			String sql = "SELECT Typ, TIME_TO_SEC(Anzahl_Stunden) AS Anzahl_Stunden, Bemerkungen, Von, Bis FROM feiertag_ausbildungstag, typ WHERE UID = ? AND feiertag_ausbildungstag.TID = typ.TID";
			if (from != null) {
				sql += " AND Von >= ?";
			}
			if (to != null) {
				sql += " AND Bis <= ?";
			}
			PreparedStatement preparedStatement = c.prepareStatement(sql);
			preparedStatement.setInt(1, calendar.getUser().getUid());
			int i = 1;
			if (from != null) {
				java.sql.Date fromSql = new java.sql.Date(from.getTime());
				preparedStatement.setDate(++i, fromSql);
			}
			if (to != null) {
				java.sql.Date toSql = new java.sql.Date(to.getTime());
				preparedStatement.setDate(++i, toSql);
			}
			ResultSet rs = preparedStatement.executeQuery();
			return getOutfallDayRangesByResultSet(rs);
		}
		
	}
	
	public class GetGroupDayRanges extends DbCallable {
		Date from;
		Date to;
		GroupCalendar calendar;
		
		public GetGroupDayRanges(Date from, Date to, GroupCalendar calendar) {
			this.from = from;
			this.to = to;
			this.calendar = calendar;
		}

		private DayRange[] getGroupDayRangesByResultSet(ResultSet rs)
		throws SQLException {
			if (!rs.next()) {
				return null;
			}
			ArrayList<DayRange> al = new ArrayList<DayRange>();
			do {
				java.sql.Date from = rs.getDate("Von");
				java.sql.Date to = rs.getDate("Bis");
				Type type = getType(rs.getString("Typ"));
				al.add(new DayRange(new Date(from.getTime()), new Date(to
						.getTime()), rs.getInt("Anzahl_Stunden"), rs
						.getString("Bemerkungen"), type));
			} while (rs.next());
			DayRange[] ga = new DayRange[al.size()];
			return al.toArray(ga);
		}
		
		public Object dbCall(Connection c) throws SQLException {
			String sql = "SELECT Typ, TIME_TO_SEC(Anzahl_Stunden) AS Anzahl_Stunden, Bemerkungen, Von, Bis FROM feiertag_ausbildungstag, typ WHERE GID = ? AND feiertag_ausbildungstag.TID = typ.TID";
			if (from != null) {
				sql += " AND Bis >= ?";
			}
			if (to != null) {
				sql += " AND Von <= ?";
			}
			PreparedStatement preparedStatement = c.prepareStatement(sql);
			preparedStatement.setInt(1, calendar.getGroup().getGid());
			int i = 1;
			if (from != null) {
				java.sql.Date fromSql = new java.sql.Date(from.getTime());
				preparedStatement.setDate(++i, fromSql);
			}
			if (to != null) {
				java.sql.Date toSql = new java.sql.Date(to.getTime());
				preparedStatement.setDate(++i, toSql);
			}
			ResultSet rs = preparedStatement.executeQuery();
			return getGroupDayRangesByResultSet(rs);
		}
		
	}
	
	public class GetHolidayRanges extends DbCallable {
		Date from;
		Date to;
		OfficialHolidayCalendar calendar;
		
		public GetHolidayRanges(Date from, Date to,
				OfficialHolidayCalendar calendar) {
			this.from = from;
			this.to = to;
			this.calendar = calendar;
		}
		
		private DayRange[] getOfficialHolidayRangesByResultSet(
				ResultSet rs) throws SQLException {
			if (!rs.next()) {
				return null;
			}
			ArrayList<DayRange> al = new ArrayList<DayRange>();
			do {
				java.sql.Date from = rs.getDate("Von");
				java.sql.Date to = rs.getDate("Bis");
				Type type = getType(rs.getString("Typ"));
				DayRange ohr = new DayRange(new Date(from
						.getTime()), new Date(to.getTime()), rs
						.getInt("Anzahl_Stunden"), rs.getString("Bemerkungen"),
						type);
				al.add(ohr);
			} while (rs.next());
			DayRange[] ga = new DayRange[al.size()];
			return al.toArray(ga);
		}

		public Object dbCall(Connection c) throws SQLException {
			String sql = "SELECT Typ, TIME_TO_SEC(Anzahl_Stunden) AS Anzahl_Stunden, Bemerkungen, Von, Bis FROM feiertag_ausbildungstag, typ WHERE GID IS NULL AND feiertag_ausbildungstag.TID = typ.TID";
			if (from != null) {
				sql += " AND Bis >= ?";
			}
			if (to != null) {
				sql += " AND Von <= ?";
			}
			PreparedStatement preparedStatement = c.prepareStatement(sql);
			int i = 0;
			if (from != null) {
				java.sql.Date fromSql = new java.sql.Date(from.getTime());
				String from = fromSql.toString();
				preparedStatement.setString(++i, from);
			}
			if (to != null) {
				java.sql.Date toSql = new java.sql.Date(to.getTime());
				String to = toSql.toString();
				preparedStatement.setString(++i, to);
			}
			ResultSet rs = preparedStatement.executeQuery();
			return getOfficialHolidayRangesByResultSet(rs);
		}
		
	}
	
	public class Save extends DbCallable {
		WorkDay day;
		UserCalendar calendar;
		
		public Save(WorkDay day, UserCalendar calendar) {
			this.day= day;
			this.calendar = calendar;
		}

		public Object dbCall(Connection c) throws SQLException {
			ResultSet rs = null;
			java.sql.Date sqlDate = new java.sql.Date(day.getDate().getTime());
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT ARID FROM arbeitstag WHERE Datum = ? AND UID = ?");
			preparedStatement.setDate(1, sqlDate);
			preparedStatement.setInt(2, calendar.getUser().getUid());
			rs = preparedStatement.executeQuery();
			boolean isUpdate = rs.next();

			String sql = " arbeitstag SET Von_Vormittag = SEC_TO_TIME(?), Bis_Vormittag = SEC_TO_TIME(?), Von_Nachmittag = SEC_TO_TIME(?), Bis_Nachmittag = SEC_TO_TIME(?), UID = ?, Datum = ?";
			if (isUpdate) {
				sql = "UPDATE" + sql + " WHERE ARID = ?";
				preparedStatement = c.prepareStatement(sql);
				preparedStatement.setInt(7, rs.getInt(1));
			} else {
				sql = "INSERT" + sql;
				preparedStatement = c.prepareStatement(sql);
			}
			Worktime[] wt = day.getWorktimes();
			preparedStatement.setInt(1, wt[0].getFrom());
			preparedStatement.setInt(2, wt[0].getTo());
			preparedStatement.setInt(3, wt[1].getFrom());
			preparedStatement.setInt(4, wt[1].getTo());
			preparedStatement.setInt(5, calendar.getUser().getUid());
			preparedStatement.setDate(6, sqlDate);
			preparedStatement.executeUpdate();
			
			return null;
		}
		
	}
	
	public class SaveDayRange extends DbCallable{
		DayRange range;
		Calendar calendar;
		
		public SaveDayRange(DayRange range, Calendar calendar) {
			this.range = range;
			this.calendar = calendar;
		}

		public Object dbCall(Connection c) throws SQLException {
			java.sql.Date sqlFromDate = new java.sql.Date(range.getDateFrom().getTime());
			java.sql.Date sqlToDate = new java.sql.Date(range.getDateTo().getTime());
			StringBuilder sb = new StringBuilder("INSERT INTO ");
			ArrayList<Object> parameters = new ArrayList<Object>();
			if(calendar instanceof OfficialHolidayCalendar) {
				sb.append("feiertag_ausbildungstag");
				sb.append(" SET GID = ?, Von = ?, Bis = ?, Anzahl_Stunden = SEC_TO_TIME(?), TID = ?, Bemerkungen = ?");
				parameters.add(null);
				parameters.add(sqlFromDate);
				parameters.add(sqlToDate);
				parameters.add(range.getAnzahlStunden());
				parameters.add(range.getType().ordinal() + 1);
				parameters.add(range.getBemerkungen());
			} else if(calendar instanceof GroupCalendar) {
				sb.append("feiertag_ausbildungstag");
				sb.append(" SET GID = ?, Von = ?, Bis = ?, Anzahl_Stunden = SEC_TO_TIME(?), TID = ?, Bemerkungen = ?");
				parameters.add(((GroupCalendar) calendar).getGroup().getGid());
				parameters.add(sqlFromDate);
				parameters.add(sqlToDate);
				parameters.add(range.getAnzahlStunden());
				parameters.add(range.getType().ordinal() + 1);
				parameters.add(range.getBemerkungen());
			} else if(calendar instanceof UserCalendar) {
				sb.append("ausfalltag");
				sb.append(" SET UID=?, Von = ?, Bis = ?, Anzahl_Stunden = SEC_TO_TIME(?), TID = ?");
				parameters.add(((UserCalendar) calendar).getUser().getUid());
				parameters.add(sqlFromDate);
				parameters.add(sqlToDate);
				parameters.add(range.getAnzahlStunden());
				parameters.add(range.getType().ordinal() + 1);
			}
			PreparedStatement preparedStatement = c.prepareStatement(sb.toString(),
					PreparedStatement.RETURN_GENERATED_KEYS);
			Object[] parametersArray = parameters.toArray();
			for(int i = 0; i < parametersArray.length; i++) {
				Object obj = parametersArray[i];
				int index = i + 1;
				if(obj == null) {
					preparedStatement.setNull(index, java.sql.Types.NULL);
				} else if(obj instanceof java.sql.Date) {
					preparedStatement.setDate(index, (java.sql.Date) parametersArray[i]);
				} else if(obj instanceof Integer) {
					preparedStatement.setInt(index, (Integer) parametersArray[i]);
				} else {
					preparedStatement.setString(index, "" + parametersArray[i]);
				}
			}
			preparedStatement.executeUpdate();
			ResultSet keys = preparedStatement.getGeneratedKeys();
			keys.next();
			int id = keys.getInt(1);
			return id;
		}
		
	}
	
	public class Visieren extends DbCallable{
		Date fromday;
		Date today;
		UserCalendar calendar;
		
		public Visieren(Date fromday, Date today, UserCalendar calendar) {
			this.fromday = fromday;
			this.today = today;
			this.calendar = calendar;
		}

		public Object dbCall(Connection c) throws Exception{
			int rs = -1;
			java.sql.Date sqlfromDate = new java.sql.Date(fromday.getTime());
			java.sql.Date sqltoDate = new java.sql.Date(today.getTime());
			String sql = "UPDATE arbeitstag SET Visiert = 1 WHERE (UID = ? AND Datum >= ? And Datum <= ?)";

			PreparedStatement preparedStatement = c.prepareStatement(sql);
			preparedStatement.setInt(1, calendar.getUser().getUid());
			preparedStatement.setDate(2, sqlfromDate);
			preparedStatement.setDate(3, sqltoDate);
			rs = preparedStatement.executeUpdate();
			if (rs < 1) {
				throw new Exception(
						"Visieren: Keine Eintraege zur Visierung vorhanden!");
			}
			return null;
		}
		
	}
	
	public class Login extends DbCallable{
		String pid;
		String password;
		
		public Login(String pid, String password) {
			this.password = password;
			this.pid = pid;
		}

		public Object dbCall(Connection c) throws Exception{
			ResultSet rs = null;
			if(c == null) {
				throw new Exception("Connection is null\n " + DbPoolPairManager.getInstance().toString());
			}
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT * FROM user WHERE PID = ? AND Passwort = MD5(?)");
			if(preparedStatement == null) {
				throw new Exception("PreparedStatement is null");
			}
			preparedStatement.setString(1, pid);
			preparedStatement.setString(2, password);
			rs = preparedStatement.executeQuery();
			if (!rs.next()) {
				return null;
			}
			return getUserByResultSet(rs, c);
		}
		
	}
	
	public class SearchUser extends DbCallable {
		String pid;
		String name;
		String firstname;
		String stringGroup;
		Group group;

		public SearchUser(String pid, String name, String firstname, Group group) {
			this.pid = pid;
			this.name = name;
			this.firstname = firstname;
			this.group = group;

			if (null != this.group) {
				stringGroup = this.group.getName();
			} else {
				stringGroup = "";
			}
		}

		public Object dbCall(Connection c) throws Exception {
			ResultSet rs = null;
			PreparedStatement preparedStatement;
			if (group == null) {

				preparedStatement = c
						.prepareStatement("SELECT * FROM user WHERE PID LIKE ? AND Nachname LIKE ? AND Vorname LIKE ?");
			} else {
				preparedStatement = c
						.prepareStatement("SELECT * FROM user, gruppe, usergruppenzuteilung WHERE PID LIKE ? "
								+ "AND Nachname LIKE ? AND Vorname LIKE ? AND Gruppenname LIKE ? "
								+ "AND gruppe.GID = usergruppenzuteilung.GID AND usergruppenzuteilung.UID = user.UID");
			}

			preparedStatement.setString(1, "%" + pid + "%");
			preparedStatement.setString(2, "%" + name + "%");
			preparedStatement.setString(3, "%" + firstname + "%");
			if (null != group) {
				preparedStatement.setString(4, "%" + stringGroup + "%");
			}
			rs = preparedStatement.executeQuery();
			ArrayList<User> user = new ArrayList<User>();
			while (rs.next()) {
				user.add(getUserByResultSet(rs, c));
			}
			User[] ua = new User[user.size()];
			return user.toArray(ua);
		}

	}
	
	public class UserByRole extends DbCallable{
		Role role;
		
		public UserByRole(Role role) {
			this.role = role;
		}

		public Object dbCall(Connection c) throws Exception{
			ResultSet rs = null;
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT * FROM user WHERE Rolle = ?");
			preparedStatement.setInt(1, role.ordinal() + 1);
			rs = preparedStatement.executeQuery();
			ArrayList<User> user = new ArrayList<User>();
			while (rs.next()) {
				user.add(getUserByResultSet(rs, c));
			}
			User[] ua = new User[user.size()];
			return user.toArray(ua);
		}
		
	}
	
	
	public class SearchGroup extends DbCallable {
		String group;
		User lehrmeister;

		public SearchGroup(String group, User lehrmeister) {
			this.group = group;
			this.lehrmeister = lehrmeister;
		}

		public Object dbCall(Connection c) throws Exception {
			ResultSet rs = null;
			StringBuffer sb = new StringBuffer(
					"SELECT * FROM gruppe WHERE 1 = 1 ");
			if (group != null) {
				sb.append("AND Gruppenname LIKE ? ");
			}
			if (lehrmeister != null) {
				sb.append("AND Lehrmeister = ? ");
			}
			PreparedStatement preparedStatement = c.prepareStatement(sb
					.toString());

			int counter = 1;
			if (group != null) {

				preparedStatement.setString(counter++, "%" + group + "%");
			}
			if (lehrmeister != null) {
				preparedStatement.setInt(counter++, lehrmeister.getUid());
			}
			rs = preparedStatement.executeQuery();

			ArrayList<Group> result = new ArrayList<Group>();
			while (rs.next()) {

				result.add(new Group(rs));
			}
			Group[] ua = new Group[result.size()];
			return result.toArray(ua);
		}

	}
	
	public class SaveUser extends DbCallable{
		User user;
		
		public SaveUser(User user) {
			this.user = user;
		}

		public Object dbCall(Connection c) throws Exception{
			boolean isUpdate = (user.getUid() != 0);

			String setPart = "user SET Vorname = ?, Nachname = ?, Anzahl_Ferientage = ?, PID = ?, Departement = ?, Berufsbildner = ?, Lehrmeister = ?, Rolle = ?";
			if(!user.isPasswortPlain()) {
				setPart = setPart + ", Passwort = ?";
			} else {
				setPart = setPart + ", Passwort = MD5(?)";
			}
			PreparedStatement preparedStatement = null;
			if (isUpdate) {
				preparedStatement = c.prepareStatement("UPDATE " + setPart
						+ " WHERE UID = ?");
				preparedStatement.setInt(10, user.getUid());
			} else {
				preparedStatement = c.prepareStatement("INSERT " + setPart,
						PreparedStatement.RETURN_GENERATED_KEYS);
			}
			preparedStatement.setString(1, user.getVorname());
			preparedStatement.setString(2, user.getName());
			preparedStatement.setFloat(3, user.getAnzahlFerientage());
			preparedStatement.setString(4, user.getPid());
			preparedStatement.setString(5, user.getDepartement());
			preparedStatement.setInt(6, user.getBerufsbildner().getUid());
			preparedStatement.setInt(7, user.getLehrmeister().getUid());
			preparedStatement.setInt(8, user.getRole().ordinal() + 1);
			preparedStatement.setString(9, user.getPasswort());
			preparedStatement.executeUpdate();

			if (!isUpdate) {
				ResultSet keys = preparedStatement.getGeneratedKeys();
				keys.next();
				int uid = keys.getInt(1);
				user.setUid(uid);
			}
			
			preparedStatement = c.prepareStatement("REPLACE INTO `usergruppenzuteilung` (`UID`, `GID`) VALUES(?, ?)");
			preparedStatement.setInt(1, user.getUid());
			preparedStatement.setInt(2, user.getGroup().getGid());
			preparedStatement.executeUpdate();
			
			return user.getUid();
		}
		
	}
	
	public class SaveGroup extends DbCallable {
		Group group;
		
		public SaveGroup(Group group) {
			this.group = group;
		}

		public Object dbCall(Connection c) throws Exception {
			boolean isUpdate = (group.getGid() != 0);

			String setPart = "gruppe SET Gruppenname = ?, Lehrmeister = ?";
			PreparedStatement preparedStatement = null;
			if (isUpdate) {
				preparedStatement = c.prepareStatement("UPDATE " + setPart
						+ " WHERE GID = ?");
				preparedStatement.setInt(3, group.getGid());
			} else {
				preparedStatement = c.prepareStatement("INSERT " + setPart,
						PreparedStatement.RETURN_GENERATED_KEYS);
			}
			preparedStatement.setString(1, group.getName());
			preparedStatement.setInt(2, group.getLehrmeister().getUid());
			preparedStatement.executeUpdate();

			if (!isUpdate) {
				ResultSet keys = preparedStatement.getGeneratedKeys();
				keys.next();
				int gid = keys.getInt(1);
				group.setGid(gid);
			}
			return group.getGid();
		}
		
	}
	
	public class GetUserByInstructor extends DbCallable {
		User instructor;
		
		public GetUserByInstructor(User instructor) {
			this.instructor = instructor;
		}

		public Object dbCall(Connection c) throws Exception {
			ResultSet rs = null;
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT * FROM user WHERE Berufsbildner = ?");
			preparedStatement.setInt(1, instructor.getUid());
			rs = preparedStatement.executeQuery();
			ArrayList<User> user = new ArrayList<User>();
			while (rs.next()) {
				user.add(getUserByResultSet(rs, c));
			}
			if (user.size() == 0) {
				return null;
			}
			User[] ua = new User[user.size()];
			return user.toArray(ua);
		}
		
	}
	
}
