package ch.il06.zeiterfassung.web;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import ch.il06.zeiterfassung.db.DayRange;
import ch.il06.zeiterfassung.db.Type;

/**
 * 
 * @author ncaspar, Fabian Hediger(A955233)
 */
public class Helper {

	/**
	 * Wandelt einen String so um, das er gefahrlos angezeigt werden kann (XSS
	 * verhindern etc.)
	 * 
	 * @param input
	 * @return Umgewandelter input
	 */
	public static String escapeHtml(String input) {
		if (input == null) {
			return "";
		}
		return input.replaceAll("&", "&amp;").replaceAll("<", "&lt;")
				.replaceAll(">", "&gt;").replaceAll("\"", "&quot;");
	}

	/**
	 * Converts a Datestring( Format dd.MM.yyyy) to a java.util.Date
	 * 
	 * @param dateString
	 * @return java.util.Date
	 * @throws ParseException
	 */
	public static Date convertStringToDate(String dateString)
			throws ParseException {
		String dateFormatString = "dd.MM.yyyy";
		SimpleDateFormat format = new SimpleDateFormat(dateFormatString);
		Date date = format.parse(dateString);
		return date;
	}

	/**
	 * Converts a Timestamp into a formated String
	 * 
	 * @param timestamp in miliseconds
	 * @param dateFormat
	 *            The format for the date, if it is null the standard format is
	 *            "dd.MM.yyyy"
	 * @return
	 */
	public static String convertTimeStampToTimeString(long timestamp, String dateFormat) {
		if("HH:mm".equals(dateFormat)){
		timestamp=timestamp/1000;
	      int sekAnteil=(int)(timestamp%60);
	      int minutenTotal=(int)((timestamp-sekAnteil)/60);
	      int minutenAnteil=minutenTotal%60;
	      int stunden=(minutenTotal-minutenAnteil)/60;
		  return ""+stunden+":" + (minutenAnteil < 10 ? "0" : "")+minutenAnteil;}
		else{
		
		  SimpleDateFormat format = new SimpleDateFormat(dateFormat);
		  String date = format.format(timestamp);
		  return date;}
	}

	/**
	 * Returns a timestamp from the actual time
	 * 
	 * @return
	 */
	public static String getActualDate(String dateFormat) {
		return convertTimeStampToTimeString(System.currentTimeMillis(), dateFormat);
	}

	/**
	 * Returns a timestamp from the actual time
	 * 
	 * @return
	 */
	public static String getActualDate() {
		return convertTimeStampToTimeString(System.currentTimeMillis(), "dd.MM.yyyy");
	}

	public static DayRange createDayRange(String paramVon, String paramBis, Type type, HttpServletRequest request)
			throws ParseException {
		String vonString = (String) request.getParameter(paramVon);
		String bisString = (String) request.getParameter(paramBis);
		System.out.println("Von:" + vonString + " bis:" + bisString);
		String bemerkung = (String) request.getParameter("bemerkung");
		Date dateFrom = Helper.convertStringToDate(vonString);
		Date dateTo = Helper.convertStringToDate(bisString);
		DayRange range = new DayRange(dateFrom, dateTo, 0, bemerkung, type);
		return range;
	}
	
	public static int halberTag(HttpServletRequest request) {
		int time;
		ResourceBundle rb = ResourceBundle.getBundle("ch.il06.zeiterfassung.properties.application");
		System.out.println("work1" + Integer.parseInt(rb.getString("worktimeForWeek")));
		System.out.println("work2" + Integer.parseInt(rb.getString("workDays")));
		double workTime = (Double.parseDouble(rb.getString("worktimeForWeek")))/(Double.parseDouble((rb.getString("workDays"))));
		if(request.getParameter("halbTag")!= null || "1".equals(request.getParameter("halbTag"))){
//			time = zeitBerechnung(dateFrom, dateTo, 4.2);
			time = (int)((workTime*3600)/2);
			System.out.println("time" + time);
		}else{
//			time = zeitBerechnung(dateFrom, dateTo, 8.4);
			time = (int)(workTime*3600);
			System.out.println("time" + time);
		}
		
		return time;
	}
	
	private static int zeitBerechnung(Date dateFrom, Date dateTo, double d) {
		int time = 0;
		
		long difference = dateFrom.getTime() - dateTo.getTime();
		long days = Math.round( (double)difference / (24. * 60.*60.*1000.) )+1;
		double temp = ((int)days)*d;
		System.out.println( "Zeit-Differenz in Tagen: " + days + " ZeitStunden - double:" + temp);
//		time = dayConvertToWorkhour(time);
		time = (int) (temp*60*60);
		System.out.println("Sekunden:" + time);
		
		return time;
	}

	public static void redirect(String destination, HttpServletRequest request, HttpServletResponse response) throws IOException {
		String path = request.getContextPath();
		String basePath = request.getScheme()+"://"+request.getServerName()+":"+request.getServerPort()+path+"/";
		response.sendRedirect(response.encodeRedirectURL(basePath + destination));
	}
}
