package ch.il06.zeiterfassung.web.controller;

import java.io.IOException;
import java.text.ParseException;
import java.util.Date;

import javax.servlet.ServletException;

import ch.il06.zeiterfassung.db.DayRange;
import ch.il06.zeiterfassung.db.Db;
import ch.il06.zeiterfassung.db.Role;
import ch.il06.zeiterfassung.db.Type;
import ch.il06.zeiterfassung.db.User;
import ch.il06.zeiterfassung.db.UserCalendar;
import ch.il06.zeiterfassung.db.WorkDay;
import ch.il06.zeiterfassung.db.Worktime;
import ch.il06.zeiterfassung.web.AbstractController;
import ch.il06.zeiterfassung.web.Helper;


/**
 * 
 * @author Fabian Hediger (A955233)
 * 
 */
public class LehrlingController extends AbstractController {
	
	
	public WorkDay[] getWorkDays(Date from, Date to, UserCalendar uc) throws Exception {
		return Db.getInstance().getWorkDays(from, to, uc);
	}

	/**
	 * Prüft ob der User eingeloggt.
	 * 
	 * @param action
	 * @author mstutz
	 * @return true wenn der User eingeloggt ist und false, wenn nicht.
	 */
	@Override
	public boolean isAllowed(String action) {
		return request.getSession().getAttribute("currentUser") != null
		&& ((User)request.getSession().getAttribute("currentUser")).getRole().equals(Role.Apprentice);
	}

	/**
	 * Zeigt die Willkommensmeldung an.
	 * 
	 * @throws javax.servlet.ServletException
	 * @throws java.io.IOException
	 */
	public void indexAction() throws ServletException, IOException {
		requestDespatcher("/lehrling/index.jsp");
	}

	/**
	 * Die Action wird aufgerufen, wenn der User die Zeit eintragen oder
	 * abspeichern will.
	 * 
	 * @throws ServletException
	 * @throws IOException
	 */
	public void eintragenAction() throws ServletException, IOException {
		if(request.getParameter("selectedDate") == null) {
			Helper.redirect("lehrling/index", request, response);
			return;
		}
		Db db = (Db) Db.getInstance();
		if ("POST".equals(request.getMethod())) {
			// TODO validate
			saveTime(db);
			Helper.redirect("lehrling/index", request, response);
			return;
		}
		requestDespatcher("/lehrling/eintragen.jsp");

	}

	/**
	 * Saves the Time into Database
	 * 
	 * @param db
	 */
	private void saveTime(Db db) {
		try {
			String vormittagvon = request.getParameter("vormittagvon");
			int vormittagvonSeconds = getSeconds(vormittagvon);
			String vormittagbis = request.getParameter("vormittagbis");
			int vormittagbisSeconds = getSeconds(vormittagbis);
			String nachmittagvon = request.getParameter("nachmittagvon");
			int nachmittagvonSeconds = getSeconds(nachmittagvon);
			String nachmittagbis = request.getParameter("nachmittagbis");
			int nachmittagbisSeconds = getSeconds(nachmittagbis);
			String dateString = (String) request.getParameter("selectedDate");
			Date date=Helper.convertStringToDate(dateString);
			WorkDay day = new WorkDay(date);
			Worktime[] worktime = new Worktime[2];
			worktime[0] = new Worktime(vormittagvonSeconds, vormittagbisSeconds);
			worktime[1] = new Worktime(nachmittagvonSeconds,
					nachmittagbisSeconds);
			day.setWorktimes(worktime);
			User user = (User) request.getSession().getAttribute("currentUser");
			UserCalendar calendar = db.getCalendarByApprentice(user);
			System.out.println("Workday" + day);
			System.out.println("UserCalendar" + calendar);
			db.save(day, calendar);
		} catch (Exception e) {
			System.out.println("LehrlingController.saveTime: " + e);
			e.printStackTrace();
		}

	}

	/**
	 * Calculates the seconds of the time
	 * 
	 * @param stringTime
	 * @return int seconds
	 */
	private int getSeconds(String stringTime) {
		String[] splittedValues = stringTime.split(":");
		int hours = Integer.parseInt(splittedValues[0]);
		int minutes = Integer.parseInt(splittedValues[1]);
		return (hours * 3600) + (minutes * 60);
	}
	
	public void ferien_eintragenAction() throws ServletException, IOException, ParseException {
		Db db = (Db) Db.getInstance();
		if ("POST".equals(request.getMethod())) {
			saveHolidayToDB(db);
			request.setAttribute("success", "Ferien erfasst!");
		}
		
		requestDespatcher("/lehrling/ferien_eintragen.jsp");
	}
	
	private void saveHolidayToDB(Db db) throws ParseException{
		try {
			User usr = (User)request.getSession().getAttribute("currentUser");
			System.out.println(request.getAttribute("halbTag"));
//			if(request.getAttribute("halbTag") != null){ //TODO FUNKTIONALITaeT TESTEN!!!
				System.out.println("Ich habs geschafft!!! *Hiro Nakamura*");
//				double tempHalfDay = 4.2*60*60;
				DayRange ferien = Helper.createDayRange("vonDatum", "bisDatum", Type.Ferien, request);
//				ferien.setAnzahlStunden((int)tempHalfDay);
				int time = Helper.halberTag(request);
				ferien.setAnzahlStunden(time);
				db.save(ferien, db.getCalendarByApprentice(usr));
//			}else{
//				DayRange ferien = Helper.createDayRange("vonDatum", "bisDatum", Type.Ferien, request, null);
//				System.out.println(ferien);
//				db.save(ferien, db.getCalendarByApprentice(usr));
//			}
		} catch (NullPointerException e) {
			e.printStackTrace();
			System.err.println("Null Pointer!");
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	public void krankheit_eintragenAction() throws ServletException, IOException, ParseException {
		Db db = (Db) Db.getInstance();
		if ("POST".equals(request.getMethod())) {
			saveIllToDB(db);
			request.setAttribute("success", "Krankheit erfasst!");
		}
		
		requestDespatcher("/lehrling/krankheit_eintragen.jsp");
	}

	private void saveIllToDB(Db db) {
		try {
			User usr = (User)request.getSession().getAttribute("currentUser");

			DayRange krankheit = Helper.createDayRange("vonDatum", "bisDatum", Type.Krankheit, request);
//			krankheit.setAnzahlStunden(timeCalculator());
			System.out.println(krankheit);
			int time = Helper.halberTag(request);
			krankheit.setAnzahlStunden(time);
			
			db.save(krankheit, db.getCalendarByApprentice(usr));

		} catch (NullPointerException e) {
			e.printStackTrace();
			System.err.println("Null Pointer!");
		} catch (Exception e){
			e.printStackTrace();
		}
	}
}
