package ch.il06.zeiterfassung.web.controller;

import java.io.IOException;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Date;

import javax.servlet.ServletException;

import ch.il06.zeiterfassung.db.DayRange;
import ch.il06.zeiterfassung.db.Db;
import ch.il06.zeiterfassung.db.Group;
import ch.il06.zeiterfassung.db.GroupCalendar;
import ch.il06.zeiterfassung.db.OfficialHolidayCalendar;
import ch.il06.zeiterfassung.db.Role;
import ch.il06.zeiterfassung.db.Type;
import ch.il06.zeiterfassung.db.User;
import ch.il06.zeiterfassung.web.AbstractController;
import ch.il06.zeiterfassung.web.Helper;

/**
 * Erledigt alle Sachen, die Relevant für die/den LehrmeisterIn sind.
 * 
 * @author Fabian Hediger (A955233)
 * @author bd
 * @author ncaspar
 * 
 */
public class LehrmeisterController extends AbstractController {

	/**
	 * Prüft ob der User eingeloggt.
	 * 
	 * @param action
	 * @author mstutz
	 * @return true wenn der User eingeloggt ist und false, wenn nicht.
	 */
	@Override
	public boolean isAllowed(String action) {
		return request.getSession().getAttribute("currentUser") != null
			&& ((User)request.getSession().getAttribute("currentUser")).getRole().equals(Role.Master);
	}

	/**
	 * Zeigt die Willkommensmeldung an.
	 * 
	 * @throws javax.servlet.ServletException
	 * @throws java.io.IOException
	 */
	public void indexAction() throws ServletException, IOException {
		requestDespatcher("/lehrmeisterIn/index.jsp");
	}
	
	public void set_sessionAction() throws Exception{
		
		String uid = request.getParameter("user");
		
		
		request.getSession().setAttribute("idUser", uid);
		
		lehrling_uebersichtAction();
	}
	
	public void lehrling_uebersichtAction() throws Exception {
		
		
		String month = request.getParameter("month");
		 String year = request.getParameter("year");
		
		request.setAttribute("month", month);
		request.setAttribute("year", year);
		
		Db db = (Db) Db.getInstance();
		
		String userId = (String) request.getSession().getAttribute("idUser");
		
		int id = Integer.parseInt(userId);
		
		User u = db.getUserById(id);
		
		request.setAttribute("user", u);
	
		requestDespatcher("/lehrmeisterIn/lehrling_uebersicht.jsp");
	}

	/**
	 * MISSING JAVA DOC TODO
	 * 
	 * @throws ServletException
	 * @throws IOException
	 * @throws Exception
	 */
	public void gruppe_waehlenAction() throws ServletException, IOException,
			Exception {

		Db db = (Db) Db.getInstance();

		setGroupsInRequest(db);
		setGroupsInRequestForLehrmeister(db);

		requestDespatcher("/lehrmeisterIn/gruppe_waehlen.jsp");

	}

	/**
	 * Nimmt die Dateneingabe und speichert diese, mittels saveGroupToDB(), in
	 * die Datenbank.
	 * 
	 * @throws ServletException
	 * @throws IOException
	 * @throws Exception
	 */
	public void gruppe_erstellenAction() throws ServletException, IOException,
			Exception {
		
		Db db = (Db) Db.getInstance();
		if ("POST".equals(request.getMethod())) {
			// TODO Validate Data
			saveGroupToDB(db);
			request.setAttribute("success", "Gruppe erstellt!");
		}

		setLehrmeisterInRequest(db);

		requestDespatcher("/lehrmeisterIn/gruppe_erstellen.jsp");
	}

	/**
	 * Erstellt einen Lehrmeister/in oder Berufsbildner/in
	 * 
	 * @throws Exception
	 * @throws IOException
	 * @throws ServletException
	 */
	public void vorgesetzter_erstellenAction() throws ServletException,
			IOException, Exception {
		Db db = (Db) Db.getInstance();
		if ("POST".equals(request.getMethod())) {
			// TODO Validate Data
			saveUserToDB(db);
			request.setAttribute("success", "Berufsbildner erstellt!");
		}

		requestDespatcher("/lehrmeisterIn/vorgesetzter_erstellen.jsp");
	}

	/**
	 * Diese Action wird aufgerufen wenn man eine Ausbildung für einen Lehrgang
	 * eintragen will.
	 * 
	 * @throws Exception
	 * @throws IOException
	 * @throws ServletException
	 */
	public void ausbildung_eintragenAction() throws ServletException,
			IOException, Exception {
		Db db = (Db) Db.getInstance();
		if ("POST".equals(request.getMethod())) {
			String art = (String) request.getParameter("artDropDown");
			Type type = null;
			if ("Ausbildung".equals(art)) {
				type = Type.Ausbildung;
			}
			if ("Schule".equals(art)) {
				type = Type.Schule;
			}else {
				type = Type.Ferien;
			}
			
			DayRange range = createDayRange("ausbildungVon", "ausbildungBis",
					type);
			GroupCalendar calendar = getGroupCalendar(db);

			db.save(range, calendar);
		}
		setGroupsInRequest(db);
		requestDespatcher("/lehrmeisterIn/ausbildung_eintragen.jsp");
	}

	/**
	 * JAVA DOC MISSING FH TODO
	 * 
	 * @param db
	 * @return
	 * @throws Exception
	 */
	private GroupCalendar getGroupCalendar(Db db) throws Exception {
		String gruppeTemp = ((String) request.getParameter("gruppeDropDown"));
		Group group = null;
		if (gruppeTemp != null) {
			int gruppeID = 0;
			try {
				gruppeID = Integer.parseInt(gruppeTemp);
			} catch (NumberFormatException nfe) {
				return null;
			}
			group = db.getGroupById(gruppeID);
		}
		if(group == null) {
			return null;
		}
		GroupCalendar calendar = db.getCalendarByGroup(group);
		return calendar;
	}

	/**
	 * Erstellt einen Lehrling
	 * 
	 * @throws Exception
	 */
	public void lehrling_erstellenAction() throws Exception {
		Db db = (Db) Db.getInstance();
		if ("POST".equals(request.getMethod())) {
			// TODO Validate Data
			String pid = saveUserToDB(db);
			request.setAttribute("success", "Lehrling " + pid + " erstellt!");
		}

		setGroupsInRequest(db);
		setBerufsbildnerInRequest(db);
		setLehrmeisterInRequest(db); // TODO lehrmeister id mitgeben an
		// searchgroup.

		requestDespatcher("/lehrmeisterIn/lehrling_erstellen.jsp");
	}

	/**
	 * Zeigt den Gruppenkalender an. Die ID der Gruppe wird über
	 * den GET-Parameter "gruppeDropDown" übergeben
	 * 
	 * @throws Exception
	 */
	public void gruppenkalenderAction() throws Exception{
		if(request.getParameter("gruppeDropDown") == null) {
			Helper.redirect("lehrmeister/index", request, response);
			return;
		}
		Db db = (Db) Db.getInstance();
		GroupCalendar calendar = getGroupCalendar(db);
		if(calendar == null) {
			Helper.redirect("lehrmeister/index", request, response);
			return;
		}
		DayRange[] ranges = getDayRangeForGroupCalendar(db, calendar);
		request.setAttribute("dayRanges", ranges);
		request.setAttribute("startDate", getDateFromRequest(false, calendar));
		request.setAttribute("group", calendar.getGroup());
		requestDespatcher("/lehrmeisterIn/gruppenkalender.jsp");

	}
	
	/**
	 * Holt einen DayRange Array aus der DB der einen Monat im Gruppen Kalander
	 * repräsentiert
	 * 
	 * @param db
	 * @param calendar
	 * @return
	 * @throws Exception
	 */
	private DayRange[] getDayRangeForGroupCalendar(Db db, GroupCalendar calendar)
			throws Exception {
		Date from = getDateFromRequest(false, calendar);
		Date to = getDateFromRequest(true, calendar);
		DayRange[] groupRanges = db.getGroupDayRanges(from, to, calendar);
		DayRange[] holidayRanges = db.getHolidayRanges(from, to, new OfficialHolidayCalendar(db));
		int length = 0;
		if(groupRanges != null) {
			length += groupRanges.length;
		}
		if(holidayRanges != null) {
			length += holidayRanges.length;
		}
		if(length != 0) {
			DayRange[] days = new DayRange[length];
			int i = 0;
			if(groupRanges != null) {
				for(DayRange range : groupRanges) {
					days[i++] = range;
				}
			}
			if(holidayRanges != null) {
				for(DayRange range : holidayRanges) {
					days[i++] = range;
				}
			}
			return days;
		}
		return null;
	}
	
	private Date getDateFromRequest(boolean lastDay, GroupCalendar calendar) throws ParseException {
		String yearString = (String) request.getParameter("yearDropDown");
		if(yearString == null) {
			yearString = Helper.convertTimeStampToTimeString(System.currentTimeMillis(), "yyyy");
		}
		String monthString = (String) request.getParameter("monthDropDown");
		if(monthString == null) {
			monthString = Helper.convertTimeStampToTimeString(System.currentTimeMillis(), "MM");
		}
		int year = Integer.parseInt(yearString);
		int monthNumber = Integer.parseInt(monthString);
		int maxDate = 1;
		if(lastDay) {
			maxDate = calendar.getMonthDay(monthNumber, year);
		}
		Date date = Helper.convertStringToDate(maxDate+"."+monthNumber+"."+year);
		return date;
	}

	/**
	 * Setzt ein User Array im Request
	 * 
	 * @param db
	 * @throws Exception
	 */
	private void setLehrmeisterInRequest(Db db) throws Exception {
		User[] lehrmeister = db.getUsersByRole(Role.Master);
		request.setAttribute("lehrmeister", lehrmeister);

	}

	/**
	 * Speichert den User in die Datenbank ab
	 */
	private String saveUserToDB(Db db) {
		try {
			String pid = (String) request.getParameter("pid");
			System.out.println("PID ist " + pid);
			String vorname = (String) request.getParameter("vorname");
			String nachname = (String) request.getParameter("nachname");

			String gruppeTemp = ((String) request
					.getParameter("gruppeDropDown"));
			Group group = null;
			if (gruppeTemp != null) {
				System.out.println("Jumped in");
				int gruppeID = Integer.parseInt(gruppeTemp);
				System.out.println("Gruppe ID variable: " + gruppeID);
				group = db.getGroupById(gruppeID);
				System.out.println("Gruppen ID gruppe: " + group.getGid());
			}

			String abteilung = (String) request.getParameter("abteilung");

			String berufsbildnerTemp = (String) request
					.getParameter("berufsbildnerInDropDown");
			User berufsbildner = null;

			if (berufsbildnerTemp != null) {
				int berufsbildnerID = Integer.parseInt(berufsbildnerTemp);
				berufsbildner = db.getUserById(berufsbildnerID);
			}
			String lehrmeisterTemp = (String) request
					.getParameter("lehrmeisterInDropDown");
			User lehrmeister = null;
			if (lehrmeisterTemp != null) {
				int lehrmeisterID = Integer.parseInt(lehrmeisterTemp);
				lehrmeister = db.getUserById(lehrmeisterID);
			}

			User user = new User();
			user.setPid(pid);
			user.setVorname(vorname);
			user.setName(nachname);
			user.setGroup(group);
			user.setBerufsbildner(berufsbildner);
			user.setDepartement(abteilung);
			user.setLehrmeister(lehrmeister);
			user.setRole(Role.Apprentice);
			user.setPasswort("7777");
			user.setAnzahlFerientage(25);
			db.save(user);
			return pid;
		} catch (Exception e) {
			e.printStackTrace();
			return "nicht";
		}

	}

	/**
	 * This Action is called when the user wants to enter a holiday
	 * 
	 * @throws IOException
	 * @throws ServletException
	 */
	public void feiertag_eintragenAction() throws ServletException, IOException {
		Db db = (Db) Db.getInstance();
		if ("POST".equals(request.getMethod())) {

			try {
				DayRange range = createDayRange("feiertagVon", "feiertagBis",
						Type.Feiertag);
				OfficialHolidayCalendar ohc = new OfficialHolidayCalendar(db);
				db.save(range, ohc);

			} catch (ParseException e) {
				System.out
						.println("LehrmeisterController.feiertag_eintragenAction ParseException");
				e.printStackTrace();
			} catch (Exception e) {
				System.out
						.println("LehrmeisterController.feiertag_eintragenAction: "
								+ e);
				e.printStackTrace();
			}
		}
		requestDespatcher("/lehrmeisterIn/feiertag_eintragen.jsp");

	}

	private DayRange createDayRange(String paramVon, String paramBis, Type type)
			throws ParseException {
			return Helper.createDayRange(paramVon, paramBis, type, request);
	}

	/**
	 * Speichert eine Gruppe in die DB.
	 * 
	 * @param db
	 */
	private void saveGroupToDB(Db db) {
		try {

			String name = (String) request.getParameter("gruppennamen");
			String lehrmeisterTemp = (String) request
					.getParameter("lehrmeisterInDropDown");

			User lehrmeister = null;
			if (lehrmeisterTemp != null) {
				int lehrmeisterID = Integer.parseInt(lehrmeisterTemp);
				lehrmeister = db.getUserById(lehrmeisterID);
			}

			Group group = new Group();
			group.setName(name);
			group.setLehrmeister(lehrmeister);
			db.save(group); // TODO MELDUNG MUSS NOCH ERSTELLT WERDEN, DASS DIE
			// GRUPPER ERSTELLT WURDE!!!
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Setzt ein Group Array im Request
	 * 
	 * @param db
	 * @throws Exception
	 * @throws ServletException
	 * @throws IOException
	 */
	private void setGroupsInRequest(Db db) throws Exception, ServletException,
			IOException {

		Group[] groups = db.searchGroup(null, null);

		request.setAttribute("groups", groups);

	}

	private void setGroupsInRequestForLehrmeister(Db db) throws Exception,
			ServletException, IOException {

		User currentUser = (User) request.getSession().getAttribute(
				"currentUser");

		Group[] groups = db.searchGroup(null, currentUser);
		request.setAttribute("groupsForLehrmeister", groups);

	}

	/**
	 * Setzt eine ArrayList<String> mit allen Berufsbildner im Request
	 * 
	 * @param db
	 * @throws Exception
	 * @throws ServletException
	 * @throws IOException
	 */
	private void setBerufsbildnerInRequest(Db db) throws Exception,
			ServletException, IOException {

		User[] berufsbildnerObjects = db.getUsersByRole(Role.Instructor);

		request.setAttribute("berufsbildner", berufsbildnerObjects);

	}

	public void user_erstellenAction() throws Exception {
		requestDespatcher("/lehrmeisterIn/user_erstellen.jsp");
	}

}
