package ch.il06.zeiterfassung.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;

public class Db implements DbInterface {

	private static DbInterface instance = null;
	private static DbConnectionManager con = null;
	final long MILLSECS_PER_DAY = 24 * 60 * 60 * 1000;

	private Db() {
		try {
			Class.forName("com.mysql.jdbc.Driver");
		} catch (ClassNotFoundException e) {
			// TODO Exception behandeln
			e.printStackTrace();
		}
		con = DbConnectionManager.getInstance();
		con.addPool("standard", new DbConnectionPool(
				"jdbc:mysql://172.16.2.67:3306/zeiterfassung", "user",
				"123456", true, 2));
	}

	public static DbInterface getInstance() {
		if (instance == null) {
			instance = new Db();
		}
		return instance;
	}

	@Override
	public Group createGroup(String name, User master) {
		Connection c = con.getConnection("standard");
		try {
			PreparedStatement preparedStatement = c.prepareStatement(
					"INSERT  gruppe SET Gruppenname = ?, Lehrmeister = ?",
					PreparedStatement.RETURN_GENERATED_KEYS);

			preparedStatement.setString(1, name);
			preparedStatement.setInt(2, master.getUid());

			ResultSet keys = preparedStatement.getGeneratedKeys();
			keys.next();
			int gid = keys.getInt(1);
			con.putConnection("standard", c);
			Group group = getGroupById(gid);
			return group;
		} catch (SQLException e) {
			return null;
		} finally {
			con.putConnection("standard", c);
		}
	}

	private Group getGroupById(int gid) {
		Connection c = con.getConnection("standard");
		try {
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT * FROM gruppe WHERE GID = ?");
			preparedStatement.setInt(1, gid);
			ResultSet rs = preparedStatement.executeQuery();
			if (!rs.next()) {
				return null;
			}
			return new Group(rs);
		} catch (SQLException e) {
			return null;
		} finally {
			con.putConnection("standard", c);
		}
	}

	private User getUserByResultSet(ResultSet rs) throws SQLException {
		User user = new User(rs);
		// System.out.println("getUserByResultSet: " + user.getUid());
		int berufsbildner = rs.getInt("Berufsbildner");
		if (berufsbildner != 0) {
			user.setBerufsbildner(getUserById(berufsbildner));
		}
		int lehrmeister = rs.getInt("Lehrmeister");
		if (lehrmeister != 0) {
			user.setLehrmeister(getUserById(lehrmeister));
		}
		Group g = getGroupByUserId(rs.getInt("UID"));
		user.setGroup(g);

		Role r = getRoleById(rs.getInt("Rolle"));
		user.setRole(r);

		return user;
	}

	private Role getRoleById(int rid) {
		Connection c = con.getConnection("standard");
		try {
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT Rollenname FROM rolle WHERE RID = ?");
			preparedStatement.setInt(1, rid);
			ResultSet rs = preparedStatement.executeQuery();
			con.putConnection("standard", c);
			if (!rs.next()) {
				return null;
			}
			String name = rs.getString(1);
			if ("Lehrling".equals(name)) {
				return Role.Apprentice;
			} else if ("Lehrmeister".equals(name)) {
				return Role.Master;
			} else if ("Berufsbildner".equals(name)) {
				return Role.Instructor;
			}
			return null;
		} catch (SQLException e) {
			con.putConnection("standard", c);
			return null;
		}
	}

	private Group getGroupByUserId(int uid) {
		// System.out.println("getGroupByUserId: " + uid);
		Connection c = con.getConnection("standard");
		try {
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT * FROM usergruppenzuteilung, gruppe WHERE gruppe.GID = usergruppenzuteilung.GID AND usergruppenzuteilung.UID = ?");
			preparedStatement.setInt(1, uid);
			ResultSet rs = preparedStatement.executeQuery();
			if (!rs.next()) {
				con.putConnection("standard", c);
				return null;
			}
			// System.out.println(uid + " hat Gruppe " +
			// rs.getString("Gruppenname"));
			Group g = new Group(rs);
			con.putConnection("standard", c);
			g.setLehrmeister(getUserById(rs.getInt("Lehrmeister")));
			return g;
		} catch (SQLException e) {
			con.putConnection("standard", c);
			return null;
		}
	}

	private User getUserById(int uid) {
		// System.out.println("getUserById: " + uid);
		Connection c = con.getConnection("standard");
		try {
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT * FROM user WHERE UID = ?");
			preparedStatement.setInt(1, uid);
			ResultSet rs = preparedStatement.executeQuery();
			if (!rs.next()) {
				return null;
			}
			con.putConnection("standard", c);
			User user = getUserByResultSet(rs);
			return user;
		} catch (SQLException e) {
			return null;
		} finally {
			con.putConnection("standard", c);
		}
	}

	@Override
	public UserCalendar getCalendarByApprentice(User user) {
		return new UserCalendar(this, user);
	}

	@Override
	public WorkDay[] getWorkDays(Date from, Date to, UserCalendar calendar) {
		Connection c = con.getConnection("standard");
		try {
			String sql = "SELECT TIME_TO_SEC(Von_Vormittag) AS Von_Vormittag, TIME_TO_SEC(Bis_Vormittag) AS Bis_Vormittag, TIME_TO_SEC(Von_Nachmittag) AS Von_Nachmittag, TIME_TO_SEC(Bis_Nachmittag) AS Bis_Nachmittag, Datum FROM arbeitstag WHERE UID = ?";
			if (from != null) {
				sql += " AND arbeitstag.Datum >= ?";
			}
			if (to != null) {
				sql += " AND arbeitstag.Datum <= ?";
			}
			PreparedStatement preparedStatement = c.prepareStatement(sql);
			preparedStatement.setInt(1, calendar.getUser().getUid());
			int i = 1;
			if (from != null) {
				java.sql.Date fromSql = new java.sql.Date(from.getTime());
				preparedStatement.setDate(++i, fromSql);
			}
			if (to != null) {
				java.sql.Date toSql = new java.sql.Date(to.getTime());
				preparedStatement.setDate(++i, toSql);
			}
			ResultSet rs = preparedStatement.executeQuery();
			con.putConnection("standard", c);
			return getWorkDaysByResultSet(rs);
		} catch (SQLException e) {
			con.putConnection("standard", c);
			return null;
		}
	}

	private WorkDay[] getWorkDaysByResultSet(ResultSet rs) throws SQLException {
		if (!rs.next()) {
			return null;
		}
		ArrayList<WorkDay> al = new ArrayList<WorkDay>();
		do {
			WorkDay wd = new WorkDay(new Date(rs.getDate("Datum").getTime()));
			Worktime[] wt = new Worktime[2];
			wt[0] = new Worktime(rs.getInt("Von_Vormittag"), rs
					.getInt("Bis_Vormittag"));
			wt[1] = new Worktime(rs.getInt("Von_Nachmittag"), rs
					.getInt("Bis_Nachmittag"));
			wd.setWorktimes(wt);
			al.add(wd);
		} while (rs.next());
		WorkDay[] wa = new WorkDay[al.size()];
		return al.toArray(wa);
	}

	@Override
	public OutfallDayRange[] getOutfallDayRanges(Date from, Date to,
			UserCalendar calendar) {
		Connection c = con.getConnection("standard");
		try {
			String sql = "SELECT Typ, Anzahl_Stunden, Bemerkungen, Von, Bis FROM feiertag_ausbildungstag, typ WHERE UID = ? AND feiertag_ausbildungstag.TID = typ.TID";
			if (from != null) {
				sql += " AND Von >= ?";
			}
			if (to != null) {
				sql += " AND Bis <= ?";
			}
			PreparedStatement preparedStatement = c.prepareStatement(sql);
			preparedStatement.setInt(1, calendar.getUser().getUid());
			int i = 1;
			if (from != null) {
				java.sql.Date fromSql = new java.sql.Date(from.getTime());
				preparedStatement.setDate(++i, fromSql);
			}
			if (to != null) {
				java.sql.Date toSql = new java.sql.Date(to.getTime());
				preparedStatement.setDate(++i, toSql);
			}
			ResultSet rs = preparedStatement.executeQuery();
			con.putConnection("standard", c);
			return getOutfallDayRangesByResultSet(rs);
		} catch (SQLException e) {
			con.putConnection("standard", c);
			return null;
		}
	}

	private OutfallDayRange[] getOutfallDayRangesByResultSet(ResultSet rs)
			throws SQLException {
		if (!rs.next()) {
			return null;
		}
		ArrayList<OutfallDayRange> al = new ArrayList<OutfallDayRange>();
		do {
			java.sql.Date from = rs.getDate("Von");
			java.sql.Date to = rs.getDate("Bis");
			Type type = getType(rs.getString("Typ"));
			al.add(new OutfallDayRange(new Date(from.getTime()), new Date(
					to.getTime()), rs.getInt("Anzahl_Stunden"), rs
					.getString("Bemerkungen"), type));
		} while (rs.next());
		OutfallDayRange[] wa = new OutfallDayRange[al.size()];
		return al.toArray(wa);
	}

	@Override
	public GroupCalendar getCalendarByGroup(Group group) {
		return new GroupCalendar(this, group);
	}

	@Override
	public GroupDayRange[] getGroupDayRanges(Date from, Date to,
			GroupCalendar calendar) {
		Connection c = con.getConnection("standard");
		try {
			String sql = "SELECT Typ, Anzahl_Stunden, Bemerkungen, Von, Bis FROM feiertag_ausbildungstag, typ WHERE GID = ? AND feiertag_ausbildungstag.TID = typ.TID";
			if (from != null) {
				sql += " AND Von >= ?";
			}
			if (to != null) {
				sql += " AND Bis <= ?";
			}
			PreparedStatement preparedStatement = c.prepareStatement(sql);
			preparedStatement.setInt(1, calendar.getGroup().getGid());
			int i = 1;
			if (from != null) {
				java.sql.Date fromSql = new java.sql.Date(from.getTime());
				preparedStatement.setDate(++i, fromSql);
			}
			if (to != null) {
				java.sql.Date toSql = new java.sql.Date(to.getTime());
				preparedStatement.setDate(++i, toSql);
			}
			ResultSet rs = preparedStatement.executeQuery();
			con.putConnection("standard", c);
			return getGroupDayRangesByResultSet(rs);
		} catch (SQLException e) {
			con.putConnection("standard", c);
			return null;
		}
	}

	private GroupDayRange[] getGroupDayRangesByResultSet(ResultSet rs)
			throws SQLException {
		if (!rs.next()) {
			return null;
		}
		ArrayList<GroupDayRange> al = new ArrayList<GroupDayRange>();
		do {
			java.sql.Date from = rs.getDate("Von");
			java.sql.Date to = rs.getDate("Bis");
			Type type = getType(rs.getString("Typ"));
			al.add(new GroupDayRange(new Date(from.getTime()), new Date(
					to.getTime()), rs.getInt("Anzahl_Stunden"), rs
					.getString("Bemerkungen"), type));
		} while (rs.next());
		GroupDayRange[] ga = new GroupDayRange[al.size()];
		return al.toArray(ga);
	}

	private Type getType(String ts) {
		if (ts.equals("Krankheit")) {
			return Type.Krankheit;
		} else if (ts.equals("Ferien")) {
			return Type.Ferien;
		} else if (ts.equals("Sonstige Abwesenheiten")) {
			return Type.SonstigeAbwesenheit;
		} else if (ts.equals("Feiertag")) {
			return Type.Feiertag;
		} else if (ts.equals("Ausbildung")) {
			return Type.Ausbildung;
		}
		return null;
	}

	@Override
	public OfficialHolidayCalendar getOfficialHolidayCalendar() {
		return new OfficialHolidayCalendar(this);
	}

	@Override
	public OfficialHolidayRange[] getHolidayRanges(Date from, Date to,
			OfficialHolidayCalendar calendar) {
		Connection c = con.getConnection("standard");
		try {
			String sql = "SELECT Typ, Anzahl_Stunden, Bemerkungen, Von, Bis FROM feiertag_ausbildungstag, typ WHERE GID IS NULL AND feiertag_ausbildungstag.TID = typ.TID";
			if (from != null) {
				sql += " AND Von >= ?";
			}
			if (to != null) {
				sql += " AND Bis <= ?";
			}
			PreparedStatement preparedStatement = c.prepareStatement(sql);
			int i = 1;
			if (from != null) {
				java.sql.Date fromSql = new java.sql.Date(from.getTime());
				preparedStatement.setDate(++i, fromSql);
			}
			if (to != null) {
				java.sql.Date toSql = new java.sql.Date(to.getTime());
				preparedStatement.setDate(++i, toSql);
			}
			ResultSet rs = preparedStatement.executeQuery();
			con.putConnection("standard", c);
			return getOfficialHolidayRangesByResultSet(rs);
		} catch (SQLException e) {
			con.putConnection("standard", c);
			return null;
		}
	}

	private OfficialHolidayRange[] getOfficialHolidayRangesByResultSet(
			ResultSet rs) throws SQLException {
		if (!rs.next()) {
			return null;
		}
		ArrayList<OfficialHolidayRange> al = new ArrayList<OfficialHolidayRange>();
		do {
			java.sql.Date from = rs.getDate("Von");
			java.sql.Date to = rs.getDate("Bis");
			Type type = getType(rs.getString("Typ"));
			al.add(new OfficialHolidayRange(new Date(from.getTime()), new Date(
					to.getTime()), rs.getInt("Anzahl_Stunden"), rs
					.getString("Bemerkungen"), type));
		} while (rs.next());
		OfficialHolidayRange[] ga = new OfficialHolidayRange[al.size()];
		return al.toArray(ga);
	}

	@Override
	@Deprecated
	public Day[] getDays(Date from, Date to, Calendar calendar) {
		throw new Error("Don't use this!");
	}

	@Override
	@Deprecated
	public void save(Day day, Calendar calendar) {
		throw new Error("Don't use this!");
	}

	@Override
	public void save(WorkDay day, UserCalendar calendar) {
		Connection c = con.getConnection("standard");
		ResultSet rs = null;
		try {
			java.sql.Date sqlDate = new java.sql.Date(day.getDate().getTime());
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT ARID FROM arbeitstag WHERE Datum = ? AND UID = ?");
			preparedStatement.setDate(1, sqlDate);
			preparedStatement.setInt(2, calendar.getUser().getUid());
			rs = preparedStatement.executeQuery();

			String sql = " arbeitstag SET Von_Vormittag = SEC_TO_TIME(?), Bis_Vormittag = SEC_TO_TIME(?), Von_Nachmittag = SEC_TO_TIME(?), Bis_Nachmittag = SEC_TO_TIME(?), UID = ?, Datum = ?";
			if (rs.next()) {
				sql = "UPDATE" + sql + " WHERE ARID = ?";
				preparedStatement = c.prepareStatement(sql);
				preparedStatement.setInt(7, rs.getInt(1));
			} else {
				sql = "INSERT" + sql;
				preparedStatement = c.prepareStatement(sql);
			}
			Worktime[] wt = day.getWorktimes();
			preparedStatement.setInt(1, wt[0].from);
			preparedStatement.setInt(2, wt[0].to);
			preparedStatement.setInt(3, wt[1].from);
			preparedStatement.setInt(4, wt[1].to);
			preparedStatement.setInt(5, calendar.getUser().getUid());
			preparedStatement.setDate(6, sqlDate);
			preparedStatement.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			con.putConnection("standard", c);
		}
	}

	@Override
	public void visieren(Date fromday, Date today, UserCalendar calendar) {
		Connection c = con.getConnection("standard");
		int rs = -1;
		try {
			java.sql.Date sqlfromDate = new java.sql.Date(fromday.getTime());
			java.sql.Date sqltoDate = new java.sql.Date(today.getTime());
			String sql = "UPDATE arbeitstag SET Visiert = 1  WHERE (UID = ? AND Datum >= ? And Datum <= ?)";

			PreparedStatement preparedStatement = c.prepareStatement(sql);
			preparedStatement.setInt(1, calendar.getUser().getUid());
			preparedStatement.setDate(2, sqlfromDate);
			preparedStatement.setDate(3, sqltoDate);
			rs = preparedStatement.executeUpdate();
			if (rs < 1) {
				throw new Exception(
						"Visieren: Keine Eintraege zur Visierung vorhanden!");
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			con.putConnection("standard", c);
		}
	}

	@Override
	public void save(OutfallDayRange day, UserCalendar calendar) {
		// TODO Auto-generated method stub

	}

	@Override
	public void save(GroupDayRange day, GroupCalendar calendar) {
		// TODO Auto-generated method stub

	}

	@Override
	public void save(OfficialHolidayRange day, OfficialHolidayCalendar calendar) {
		// TODO Auto-generated method stub

	}

	@Override
	public User login(String pid, String password) {
		Connection c = con.getConnection("standard");
		ResultSet rs = null;
		try {
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT * FROM user WHERE PID = ? AND Passwort = MD5(?)");
			preparedStatement.setString(1, pid);
			preparedStatement.setString(2, password);
			rs = preparedStatement.executeQuery();
			if (!rs.next()) {
				return null;
			}
			con.putConnection("standard", c);
			return getUserByResultSet(rs);
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		} finally {
			con.putConnection("standard", c);
		}
	}

	@Override
	public Group[] searchGroup(String group) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public User[] searchUser(String pid, String name, String firstname,
			Group group) {
		Connection c = con.getConnection("standard");
		ResultSet rs = null;
		try {
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT * FROM user WHERE PID LIKE ? AND Nachname LIKE ? AND Vorname LIKE ?");
			preparedStatement.setString(1, "%" + pid + "%");
			preparedStatement.setString(2, "%" + name + "%");
			preparedStatement.setString(3, "%" + firstname + "%");
			rs = preparedStatement.executeQuery();
			ArrayList<User> user = new ArrayList<User>();
			con.putConnection("standard", c);
			while (rs.next()) {
				user.add(getUserByResultSet(rs));
			}
			User[] ua = new User[user.size()];
			return user.toArray(ua);
		} catch (SQLException e) {
			con.putConnection("standard", c);
			return null;
		}
	}

	@Override
	public int save(User user) {
		Connection c = con.getConnection("standard");
		try {
			boolean isUpdate = (user.getUid() == 0);

			String setPart = "user SET Vorname = ?, Nachname = ?, Anzahl_Ferientage = ?, PID = ?, Departement = ?, Berufsbildner = ?, Lehrmeister = ?";
			PreparedStatement preparedStatement = null;
			if (isUpdate) {
				preparedStatement = c.prepareStatement("UPDATE " + setPart
						+ " WHERE UID = ?");
				preparedStatement.setInt(8, user.getUid());
			} else {
				preparedStatement = c.prepareStatement("INSERT " + setPart,
						PreparedStatement.RETURN_GENERATED_KEYS);
			}
			preparedStatement.setString(1, user.getVorname());
			preparedStatement.setString(2, user.getName());
			preparedStatement.setFloat(3, user.getAnzahlFerientage());
			preparedStatement.setString(4, user.getPid());
			preparedStatement.setString(5, user.getDepartement());
			preparedStatement.setInt(6, user.getBerufsbildner().getUid());
			preparedStatement.setInt(7, user.getLehrmeister().getUid());
			preparedStatement.executeUpdate();

			if (!isUpdate) {
				ResultSet keys = preparedStatement.getGeneratedKeys();
				keys.next();
				int uid = keys.getInt(1);
				return uid;
			}
			return user.getUid();
		} catch (SQLException e) {
			return 0;
		} finally {
			con.putConnection("standard", c);
		}
	}

	@Override
	public int save(Group group) {
		Connection c = con.getConnection("standard");
		try {
			boolean isUpdate = (group.getGid() == 0);

			String setPart = "gruppe SET Gruppenname = ?, Lehrmeister = ?";
			PreparedStatement preparedStatement = null;
			if (isUpdate) {
				preparedStatement = c.prepareStatement("UPDATE " + setPart
						+ " WHERE GID = ?");
				preparedStatement.setInt(3, group.getGid());
			} else {
				preparedStatement = c.prepareStatement("INSERT " + setPart,
						PreparedStatement.RETURN_GENERATED_KEYS);
			}
			preparedStatement.setString(1, group.getName());
			preparedStatement.setInt(2, group.getLehrmeister().getUid());
			preparedStatement.executeUpdate();

			if (!isUpdate) {
				ResultSet keys = preparedStatement.getGeneratedKeys();
				keys.next();
				int uid = keys.getInt(1);
				return uid;
			}
			return group.getGid();
		} catch (SQLException e) {
			return 0;
		} finally {
			con.putConnection("standard", c);
		}
	}

	@Override
	public User[] getUserByInstructor(User instructor) {
		// System.out.println("getUserByInstructor: " + instructor.getUid());
		Connection c = con.getConnection("standard");
		ResultSet rs = null;
		try {
			PreparedStatement preparedStatement = c
					.prepareStatement("SELECT * FROM user WHERE Berufsbildner = ?");
			preparedStatement.setInt(1, instructor.getUid());
			rs = preparedStatement.executeQuery();
			ArrayList<User> user = new ArrayList<User>();
			con.putConnection("standard", c);
			while (rs.next()) {
				user.add(getUserByResultSet(rs));
			}
			if (user.size() == 0) {
				return null;
			}
			User[] ua = new User[user.size()];
			return user.toArray(ua);
		} catch (SQLException e) {
			con.putConnection("standard", c);
			return null;
		}
	}
}
