package ch.il06.zeiterfassung.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Vector;

/**
 *
 * @author ncaspar
 */
public class DbConnectionPool {

    private Vector<Connection> freeConnections = new Vector<Connection>();
    private Vector<Connection> busyConnections = new Vector<Connection>();
    private int maxConnections;
    private String dbUrl,  username,  password;
    private boolean autoCommit;

    public DbConnectionPool(String dbUrl, String username, String password, boolean autoCommit, int maxConnections) {
        if (maxConnections <= 0) {
            throw new IllegalArgumentException("maxConnections muss grösser als 0 sein.");
        }
        this.maxConnections = maxConnections;
        this.dbUrl = dbUrl;
        this.username = username;
        this.password = password;
        this.autoCommit = autoCommit;
    }

    public Connection getConnection() throws SQLException {
        cleanUp();
        if (freeConnections.size() == 0) {
            if (busyConnections.size() >= maxConnections) {
                throw new IllegalStateException("No more connections available");
            }
            addConnection();
        }
        Connection con = freeConnections.firstElement();
        freeConnections.remove(con);
        busyConnections.add(con);
        return con;
    }

    public void putConnection(Connection con) throws SQLException {
        if (busyConnections != null && con != null && !con.isClosed() && busyConnections.contains(con)) {
            con.setAutoCommit(autoCommit);
            con.setReadOnly(false);
            busyConnections.remove(con);
            freeConnections.add(con);
        }
    }

    private void addConnection() throws SQLException {
        Connection con;
        con = DriverManager.getConnection(dbUrl, username, password);
        con.setAutoCommit(autoCommit);
        freeConnections.add(con);
    }

    private void cleanUp() throws SQLException {
        for (Connection con : busyConnections) {
            if (con.isClosed()) {
                busyConnections.remove(con);
            }
        }
        for (Connection con : freeConnections) {
            if (con.isClosed()) {
                freeConnections.remove(con);
            }
        }
    }

    @Override
    public String toString() {
        return "Free connections: " + freeConnections.size() + "\n" +
               "Busy connections: " + busyConnections.size();
    }
}
