package ch.il06.zeiterfassung.gui;

import java.awt.Color;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import ch.il06.zeiterfassung.db.Db;
import ch.il06.zeiterfassung.db.DbInterface;
import ch.il06.zeiterfassung.db.User;
import ch.il06.zeiterfassung.db.UserCalendar;
import ch.il06.zeiterfassung.db.WorkDay;
import ch.il06.zeiterfassung.db.Worktime;

@SuppressWarnings("serial")
public class AddTime extends JDialog implements ActionListener,
		DocumentListener {

	JPanel morningPanel = new JPanel();
	JPanel afternoonPanel = new JPanel();
	JPanel background = new JPanel();
	JTextField morningFrom = new JTextField(3);
	JTextField morningTo = new JTextField(3);
	JTextField afternoonFrom = new JTextField(3);
	JTextField afternoonTo = new JTextField(3);
	JLabel from = new JLabel("(HH:MM) Von: ");
	JLabel to = new JLabel("Bis: ");
	JLabel from2 = new JLabel("(HH:MM) Von: ");
	JLabel to2 = new JLabel("Bis: ");
	JButton save = new JButton("Speichern");
	JButton cancel = new JButton("Abbrechen");
	String match = "^(([0-2][0-9]):([0-5][0-9]))$";
	DbInterface db = Db.getInstance();
	CalendarField cf;

	int hours;
	int minutes;

	int day, month, year;

	User user;

	public AddTime(int day, int month, int year, Frame owner, User user, CalendarField cf) {

		super(owner, "-", true);

		this.day = day;
		this.month = month;
		this.year = year;
		this.user = user;
		this.cf = cf;

		this.setTitle(Integer.toString(this.day) + "."
				+ Integer.toString(this.month) + "."
				+ Integer.toString(this.year));

		layoutFormat();
		addComponents();
		this.setVisible(true);

	}

	@SuppressWarnings("static-access")
	private void layoutFormat() {

		this.setSize(505, 160);
		this.setDefaultCloseOperation(this.DISPOSE_ON_CLOSE);
		this.setLayout(new GridLayout(2, 2));
		this.add(morningPanel);
		this.add(afternoonPanel);
		this.setResizable(false);
		save.addActionListener(this);
		cancel.addActionListener(this);

	}

	private void addComponents() {

		TitledBorder vormittag = new TitledBorder("Vormittag");
		morningPanel.setBorder(vormittag);

		TitledBorder nachmittag = new TitledBorder("Nachmittag");
		afternoonPanel.setBorder(nachmittag);

		morningPanel.add(from);
		morningPanel.add(morningFrom);
		morningPanel.add(to);
		morningPanel.add(morningTo);

		afternoonPanel.add(from2);
		afternoonPanel.add(afternoonFrom);
		afternoonPanel.add(to2);
		afternoonPanel.add(afternoonTo);

		save.setEnabled(false);

		morningFrom.getDocument().addDocumentListener(this);
		morningTo.getDocument().addDocumentListener(this);
		afternoonFrom.getDocument().addDocumentListener(this);
		afternoonTo.getDocument().addDocumentListener(this);

		this.add(save);
		this.add(cancel);

	}

	@Override
	public void actionPerformed(ActionEvent e) {

		if (e.getActionCommand().equals("Speichern")) {

			UserCalendar cal = db.getCalendarByApprentice(user);
			WorkDay wd = new WorkDay(new Date((year - 1900), (month - 1), day));
			Worktime wts[] = new Worktime[2];

			int morningSeconds1 = getSeconds(morningFrom.getText());
			int morningSeconds2 = getSeconds(morningTo.getText());
			int afternoonSeconds1 = getSeconds(afternoonFrom.getText());
			int afternoonSeconds2 = getSeconds(afternoonTo.getText());

			wts[0] = new Worktime(morningSeconds1, morningSeconds2);
			wts[1] = new Worktime(afternoonSeconds1, afternoonSeconds2);

			wd.setWorktimes(wts);

			db.save(wd, cal);

			cf.updateLabel(Color.RED, "Arbeiten");
			this.dispose();

		} else if (e.getActionCommand().equals("Abbrechen")) {

			this.dispose();

		}
	}

	private int getSeconds(String time) {

		String[] timeArr = new String[1];
		timeArr = time.split(":");

		hours = Integer.parseInt(timeArr[0]);
		minutes = Integer.parseInt(timeArr[1]);

		hours = hours * 3600;
		minutes = minutes * 60;

		int seconds = hours + minutes;

		return seconds;
	}

	@Override
	public void changedUpdate(DocumentEvent e) {

	}

	private boolean contentCorrect() {

		if (morningFrom.getText().matches(match)
				&& morningTo.getText().matches(match)
				&& afternoonFrom.getText().matches(match)
				&& afternoonTo.getText().matches(match)) {
			// TODO - if the first digit is a 2, the second digit should be < 5.
			return true;
		} else {

			return false;
		}

	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		save.setEnabled(contentCorrect());
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		save.setEnabled(contentCorrect());
	}

	// test only
	// public static void main(String[] args) {
	// new AddTime(2, 7, 1991, null, null);
	// }
}
